/**************************************************************
	Gakken Otonano kagaku vol.24
	4bit microcomputer GMC-4 serial interface
	 for PIC16F648A/628(A)
 **************************************************************
	author air_variable
	http://www.ne.jp/asahi/air/variable/
	version 1.05a
	Tools:
	MPLAB v8.33
	Hitech PIC C pro9.65pl1
---------------------------------------------------------------
Date	:Time	Version	Build	Note
20090818:0700	1.00	---		begin the program
20090820:2200	1.01	---		command fix and correspond XON/XOF and Hardware Flow
20090822:0138	1.02	---		command fix
20090822:0900	1.03	---		minor fix (help instruction)
20090823:1300	1.04	---		add remark command change baud rate (1200 bps)
20090827:0030	1.05a	---		Fix behave increment command

---------------------------------------------------------------
	Pin assignment
	1:RA2:O:KEY_ROW2
	2:RA3:O:KEY_ROW3
	3:RA4:I:(not used)
	4:MCLR:I:PICKIT2-1(W/10kPU)
	5:VSS:GND
	6:RB0:O:KEY_nCOL0
	7:RB1:I:Receive serial
	8:RB2:O:Transmit serial
	9:RB3:O:KEY_nCOL1

	18:RA1:O:KEY_ROW1
	17:RA0:O:KEY_ROW0
	16:RA7:O:Remort_ON
	15:RA6:O:KEY_ROW4
	14:VDD:Power 3 to 5 Volt
	13:RB7:O:RTS_out (pickit2) ***1.01
	12:RB6:exclude use for pickit2
	11:RB5:O:KEY_nCOL3
	10:RB4:O:KEY_nCOL2
	
	Key table
	  ROW 0 1 2 3 4
	nCOL0 C D E F A-SET
	nCOL1 8 9 A B INCR
	nCOL2 4 5 6 7 RUN
	nCOL3 0 1 2 3 RESET

	extra command P or p pause 0.5sec	
	extra command ; Remark(until 0x0d)	***1.04
	
	0`9  0`9 
	A`F  A`F 
	a`  A`F 
	S,s,:(p̃R)  ADR SET
	I,i,16iL[ Xy[Xors(0x0d)  INCR
		s͒O16iłꍇINCRs܂B
		p̃Rȍ~̃Xy[X͎̐s܂Ŗ܂B ***1.05
	G,g,R,r,!(p̃GNXN[V)  RUN ***1.02
	/,x,X(p̃XbV)  RESET  ***1.02

		
	communication settings
	baud:1200pps 8bit flow:xon/off or hardware CTS stop:1bit
	ӎ
	肱ڂHɔ邽߁A{[[g1200ɕύXɂ܂B
	USBVA̐\ȂǂɂāACɑꍇd~ꍇ܂B
	̏ꍇ̓u[NMESCR[hȂǂ𑗏oĂ݂ĂB
	t[XON/XOFFƃn[hEFA䂪o܂A
	n[hEFȀꍇDSR-DSRԁAPIC12s-CTS2ӏڑKv܂B
	Z~R(;)gȍ~ACR(0x0d)܂ŒߍsƂĂ݂Ȃ܂B
	pM0.5bԓ삪~܂B
	
	test program
	;LED̃AiO_
	x
	00: a 1 8 0 4 a 0 e 0 5 9 2 f 1 2 f
	10: 0 a 8 0 1 5 9 e f 1 6 a 2 8 1 6
	20: f 2 7 4 f 0 5 4 a 1 5 a 0 c 0 f
	30: 3 c 8 2 6 f 4 6 4 f 0 5 8 2 7 f
	40: 4 6 4 f 0 5 a 1 5 e 4 4 f 0 5 
	x1rx1r
	;ŌオRUNResetĂ̂́AGMC-4oÔ߂ł
	
**************************************************************/

#include <htc.h>
#include "sci.h"

#define MHz 000000
#define _XTAL_FREQ 4MHz



//Key control settings
#define	Set_key_column_0	TRISB =	TRISB;\
							PORTB &=  0b11111110;	//RB0=0
#define	Set_key_column_1	TRISB =	TRISB;\
							PORTB &=  0b11110111;	//RB3=0
#define	Set_key_column_2	TRISB =	TRISB;\
							PORTB &=  0b11101111;	//RB4=0
#define	Set_key_column_3	TRISB =	TRISB;\
							PORTB &=  0b11011111;	//RB5=0

#define Set_key_row_0		TRISA =	TRISA;\
							PORTA |=  0b00000001;	//RA0=1
#define Set_key_row_1		TRISA =	TRISA;\
							PORTA |=  0b00000010;	//RA1=1
#define Set_key_row_2		TRISA =	TRISA;\
							PORTA |=  0b00000100;	//RA2=1
#define Set_key_row_3		TRISA =	TRISA;\
							PORTA |=  0b00001000;	//RA3=1
#define Set_key_row_4		TRISA =	TRISA;\
							PORTA |=  0b01000000;	//RA6=1

#define	Release_key			TRISA =	TRISA;\
							PORTA &=  0b10110000;\
							TRISB =	TRISB;\
							PORTB |=  0b00111001;
#define Remort_ON			RA7 = 1;
#define Remort_OFF			RA7 = 0;


//Key timing settings
#define	Key_release_time	40	//ms
#define	Key_input_time		40	//ms
#define	Pause_time			50	//x10ms

//receive setting
#define USART_SPEED	1200	//speed set	//***1.04
#define MAX_BUFF	80		//RX ring buffer size
#define X_ON	0x11		//***1.01
#define X_OFF	0x13		//***1.01
#define RTS_ON	RB7=0;		//***1.01
#define RTS_OFF	RB7=1;		//***1.01

//eeprom recognition
#define STX	0x02
#define ETX	0x03

//Grobal settings

extern void delay_10ms(unsigned int temp);

//Receive FIFO parameter
volatile unsigned char Buffer_FIFO[MAX_BUFF];
unsigned char FIFO_write_pointer=0;
unsigned char FIFO_read_pointer=0;
unsigned char FIFO_datasize=0;

//Other
unsigned char Before_HEXNUM=0;		//1:O16i 0:ȊO
unsigned char FLAG_remark_line=0;	//1:̍sߍs
unsigned char FLAG_INCR_disable=0;	//1:INCRj~ ***1.05


//__CONFIG(0x3F02);
__CONFIG(
	UNPROTECT & LVPDIS & BORDIS & MCLRDIS & PWRTEN & WDTDIS & INTIO
	);

void interrupt handler( void ){		//interrupt handle
	unsigned char temp;
	unsigned char data;
	//Handle RS232 Receive interrupt
	if( RCIF == 1 ){
		//receive data
		data = sci_GetByte();
		//set to FIFO
		RCIF = 0; //clear RS232 receive interrupt bit
		if(FIFO_datasize<MAX_BUFF){	
			Buffer_FIFO[FIFO_write_pointer++]=data;
			if ( FIFO_write_pointer == MAX_BUFF ) FIFO_write_pointer=0;
			FIFO_datasize++;
		}
	}
}



void delay_10ms(unsigned int temp){	//10ms multiple x times
	while(temp--){
		__delay_ms(10);
	}	
}
	

void Key_entry(unsigned char temp){
	//ߍs̔菈
	if (FLAG_remark_line==1){
		if ( temp == 0x0d ){
			FLAG_remark_line=0;
			Before_HEXNUM=0;
			return;
		}else return;	
	}

	//|[YR}h   ***1.04
	if ((temp=='p')||(temp=='P')){
		delay_10ms(Pause_time);
		Before_HEXNUM=0;	//***1.01
		return;
	}	


	//L[V[gւ̃ANZXJn
	Remort_ON;
	switch ( temp ){
		case ';':	//remark line
			FLAG_remark_line=1;	//remark line until find 0x0d(CR) cord
			Remort_OFF;
			return;


		case '?'://help information
			sci_puts("\n\rGMC-4 serial interface ver1.05a (C)2009 air_variable\n\r");
			sci_puts("[ASET ]	'S' or ':'\n\r");
			sci_puts("[INCR ]	'I' , space or Hexnumber+CR code\n\r");
			sci_puts("[RUN  ]	'G' , 'R' or '!'\n\r");
			sci_puts("[RESET]	'X' or '/'\n\r");

			sci_puts("Pause0.5sec 'P' \n\r");
			sci_puts("Remark line ';' until CR \n\r\n\r");
			return;

			//Set_key_row
		case '0':
		case '4':
		case '8':
		case 'c':
		case 'C':
			Set_key_row_0;
			FLAG_INCR_disable=0;	//***1.05
			Before_HEXNUM=1;	//***1.01
			break;
		case '1':
		case '5':
		case '9':
		case 'd':
		case 'D':
			Set_key_row_1;
			FLAG_INCR_disable=0;	//***1.05
			Before_HEXNUM=1;	//***1.01
			break;
		case '2':
		case '6':
		case 'a':
		case 'A':
		case 'e':
		case 'E':
			Set_key_row_2;
			FLAG_INCR_disable=0;	//***1.05
			Before_HEXNUM=1;	//***1.01
			break;
		case '3':
		case '7':
		case 'b':
		case 'B':
		case 'f':
		case 'F':
			Set_key_row_3;
			FLAG_INCR_disable=0;	//***1.05
			Before_HEXNUM=1;	//***1.01
			break;
		case 's':	// [ASET]
		case 'S':	// [ASET]


		case 'g':	// [RUN]
		case 'G':	// [RUN]
		case 'r':	// [RUN]
		case 'R':	// [RUN]
		case '!':	// [RUN]

		case 'x':	// [RESET]
		case 'X':	// [RESET]
		case '/':	// [RESET]
			Set_key_row_4;
			FLAG_INCR_disable=0;	//***1.05
			Before_HEXNUM=0;	//***1.01
			break;

		case ':':	// [ASET]
			FLAG_INCR_disable=1;	//***1.05
			Set_key_row_4;
			Before_HEXNUM=0;	//***1.01
			break;

		case 'i':	// [INCR]
		case 'I':	// [INCR]
		case ' ':	// [INCR]
			if( FLAG_INCR_disable==0 ) {
				Set_key_row_4;
				Before_HEXNUM=0;
				break;
			}else{
				Remort_OFF;
				return;
			}

		case 0x0d:	// CR KEY [INCR] and before hexnumber  ***1.01 ***1.03
			FLAG_INCR_disable=0;	//***1.05
			if( Before_HEXNUM==1 ) {
				Set_key_row_4;
				Before_HEXNUM=0;
				break;
			}else{
				Remort_OFF;
				return;
			}
		default:
			break;
	}		

	switch ( temp ){	//Set_key_column
		case '0':
		case '1':
		case '2':
		case '3':

		case '/':	// [RESET]
		case 'x':	// [RESET]
		case 'X':	// [RESET]

			Set_key_column_3;
			break;

		case '4':
		case '5':
		case '6':
		case '7':

		case 'g':	// [RUN]
		case 'G':	// [RUN]
		case 'r':	// [RUN]
		case 'R':	// [RUN]
		case '!':	// [RUN]
			Set_key_column_2;
			break;

		case '8':
		case '9':
		case 'a':
		case 'A':
		case 'b':
		case 'B':

		case 'i':	// [INCR]
		case 'I':	// [INCR]
		case ' ':	// [INCR]
			Set_key_column_1;
			break;

		case 0x0d:	// [INCR] and before hexnumber (already checked)***1.01 ***1.03
			Set_key_column_1;
			break;

		case 'c':
		case 'd':
		case 'e':
		case 'f':

		case 'C':
		case 'D':
		case 'E':
		case 'F':

		case 's':	// [ASET]
		case 'S':	// [ASET]
		case ':':	// [ASET]
			Set_key_column_0;
			break;

		default:
			Release_key	
			Remort_OFF;
//			Before_HEXNUM=0;	//***1.01

			return;
	}

	__delay_ms(Key_input_time);
	Release_key;
	__delay_ms(Key_release_time);
	Remort_OFF;	// The internal key able to use it.

	return;
}		

void sci_valueout(unsigned char temp){ //debug only
	sci_PutByte(((temp / 100)%10)+0x30);//100
	sci_PutByte(((temp /  10)%10)+0x30);//10
	sci_PutByte(((temp      )%10)+0x30);//1
	return;
}
		

//------------------main
main(){
	unsigned char temp,i,FLAG_BUFF_FULL=0;
	
;

//	Port initalize
//		  	  76543210
	PORTA = 0b00000000;	//Port Initalize
	PORTB = 0b10111001;	//Port Initalize
	TRISA = 0b00110000; //
	TRISB = 0b00000110; //
	CMCON = 0x07;		//PORTA is all digital I/O

	delay_10ms(50);	//0.5sec delay


//ʐM
	sci_Init( USART_SPEED,SCI_EIGHT);// Baud set and Bit set
	CREN =0;	//communication stop
	while (RCIF ==1){
		temp = sci_GetByte();		//Dummy read
	}
	//sci check
	if (sci_CheckOERR()) { //sci error
		while (!TRMT);
		CREN=1;
	}

	RCIE = 1;		//receive interrupt on
	PEIE = 1;		//Peripheral interrupt enable
	GIE  = 1;		//Global interrupt enable
	CREN = 1;		//communication stop

//Flow control ***1.01
	sci_PutByte(X_ON);
	RTS_ON;

	
	while(1){		//Loop forever 
		//FIFO buff monitoring ***1.01
		if ((FIFO_datasize==0) && (FLAG_BUFF_FULL==1)){
			RTS_ON;			//hardware flow on
			sci_PutByte(X_ON);//communication enable
			FLAG_BUFF_FULL=0;//reset flag

		}else if ( (FIFO_datasize > (MAX_BUFF*2/3)) && (FLAG_BUFF_FULL==0)){
			RTS_OFF;		//hardware flow off
			sci_PutByte(X_OFF);//communication disable
			FLAG_BUFF_FULL=1;//set com disable flag
		}

// Reading FIFO and exercise
		if (FIFO_datasize !=0 ){
			temp = Buffer_FIFO[FIFO_read_pointer++];
			if ( FIFO_read_pointer == MAX_BUFF ) FIFO_read_pointer=0;
			FIFO_datasize--;
			sci_PutByte(temp);	//echo back
			Key_entry( temp );	//key entry
		}
	}
}
